from dataharvest.purifier.baidu_purifier import BaiJiaHaoPurifier
from dataharvest.purifier.baidu_purifier import BaiduBaikePurifier
from dataharvest.purifier.bilibili_purifier import BilibiliPurifier
from dataharvest.purifier.common_purifier import CommonPurifier
from dataharvest.purifier.purifier import AutoPurifier, BasePurifier
from dataharvest.purifier.qqnew_purifier import QQNewPurifier
from dataharvest.purifier.sll_purifier import SllPurifier
from dataharvest.purifier.sobaike_purifier import SoBaikePurifier
from dataharvest.purifier.sogou_purifier import SogouBaiKePurifier
from dataharvest.purifier.sohu_purifier import SohuPurifier
from dataharvest.purifier.toutiao_purifier import ToutiaoPurifier
from dataharvest.purifier.wangyi_purifier import WangYiPurifier
from dataharvest.purifier.wechat_purifier import WechatPurifier
from dataharvest.purifier.xiaohongshu_purifier import XiaoHongShuPurifier

__all__ = [
    "BasePurifier",
    "CommonPurifier",
    "AutoPurifier",
    "BaiduBaikePurifier",
    "SohuPurifier",
    "SllPurifier",
    "BaiJiaHaoPurifier",
    "WangYiPurifier",
    "SogouBaiKePurifier",
    "BilibiliPurifier",
    "QQNewPurifier",
    "SoBaikePurifier",
    "ToutiaoPurifier",
    "WechatPurifier",
    "XiaoHongShuPurifier"
]



import hashlib
import json
import re
from typing import Optional

import httpx
from httpx import Response
from parsel import Selector
from playwright.async_api import async_playwright
from playwright.sync_api import sync_playwright

from dataharvest.schema import Document
from dataharvest.spider.base import SpiderConfig
from dataharvest.spider.spider import BaseSpider


class MaFengWoSpider(BaseSpider):
    def __init__(self, config: Optional[SpiderConfig] = None):
        super().__init__(config)
        self.client = httpx.Client(**BaseSpider.convert_2_httpx_client_arg(config))
        self.a_client = httpx.AsyncClient(
            **BaseSpider.convert_2_httpx_client_arg(config))
        self._config = self._merge_config(config)

        if not self._config.headers:
            self._config.headers = {
                "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            }

    def match(self, url: str) -> bool:
        return "/www.mafengwo.cn/i/" in url

    def crawl(self, url: str, config: Optional[SpiderConfig] = None) -> Document:
        config = self._merge_config(config)

        # 第一次请求，获取__jsluid_s，并解析__jsl_clearance_s
        first_resp = self.client.get(url, headers=config.headers)

        jsluid_s = first_resp.cookies["__jsluid_s"]

        # 用正则表达式匹配出需要的部分
        first_jsl_clearance_s_raw = re.findall(
            "cookie=(.*?);location", first_resp.text
        )[0]

        # 反混淆、分割出cookie的部分
        with sync_playwright() as playwright:
            browser = playwright.chromium.launch()
            page = browser.new_page()
            first_jsl_clearance_s = (
                page.evaluate(first_jsl_clearance_s_raw).split(";")[0].split("=")[1]
            )

        # 第二次请求，解析新的__jsl_clearance_s
        second_cookies = {
            "__jsluid_s": jsluid_s,
            "__jsl_clearance_s": first_jsl_clearance_s,
        }
        second_resp = self.client.get(
            url, headers=config.headers, cookies=second_cookies
        )

        second_jsl_clearance_s = handle_go(second_resp.text)

        # 第三次请求，获取真实页面内容
        third_cookies = {
            "__jsluid_s": jsluid_s,
            "__jsl_clearance_s": second_jsl_clearance_s,
        }
        third_resp = self.client.get(url, headers=config.headers, cookies=third_cookies)

        final_content = handle_final_content(third_resp)

        document = Document(url=url, metadata={}, page_content=final_content)
        return document

    async def a_crawl(
            self, url: str, config: Optional[SpiderConfig] = None
    ) -> Document:
        config = self._merge_config(config)

        # 第一次请求，获取__jsluid_s，并解析__jsl_clearance_s
        first_resp = await self.a_client.get(url, headers=config.headers)

        jsluid_s = first_resp.cookies["__jsluid_s"]

        # 用正则表达式匹配出需要的部分
        first_jsl_clearance_s_raw = re.findall(
            "cookie=(.*?);location", first_resp.text
        )[0]

        # 反混淆、分割出cookie的部分
        async with async_playwright() as playwright:
            browser = await playwright.chromium.launch()
            page = await browser.new_page()
            first_jsl_clearance_s_js = await page.evaluate(first_jsl_clearance_s_raw)
            first_jsl_clearance_s = first_jsl_clearance_s_js.split(";")[0].split("=")[1]

        # 第二次请求，解析新的__jsl_clearance_s
        second_cookies = {
            "__jsluid_s": jsluid_s,
            "__jsl_clearance_s": first_jsl_clearance_s,
        }

        second_resp = await self.a_client.get(
            url, headers=config.headers, cookies=second_cookies
        )

        second_jsl_clearance_s = handle_go(second_resp.text)

        # 第三次请求，获取真实页面内容
        third_cookies = {
            "__jsluid_s": jsluid_s,
            "__jsl_clearance_s": second_jsl_clearance_s,
        }
        third_resp = await self.a_client.get(
            url, headers=config.headers, cookies=third_cookies
        )

        final_content = handle_final_content(third_resp)

        document = Document(url=url, metadata={}, page_content=final_content)
        return document


def handle_go(second_resp_text: str) -> str:
    """
    处理JS混淆代码逻辑
    :param second_resp_text:
    :return:
    """

    second_jsl_clearance_s = None
    ha = None

    go = json.loads(re.findall(r"};go\((.*?)\)</script>", second_resp_text)[0])
    for i in range(len(go["chars"])):
        for j in range(len(go["chars"])):
            values = go["bts"][0] + go["chars"][i] + go["chars"][j] + go["bts"][1]
            if go["ha"] == "md5":
                ha = hashlib.md5(values.encode()).hexdigest()
            elif go["ha"] == "sha1":
                ha = hashlib.sha1(values.encode()).hexdigest()
            elif go["ha"] == "sha256":
                ha = hashlib.sha256(values.encode()).hexdigest()

            if ha == go["ct"]:
                second_jsl_clearance_s = values

    return second_jsl_clearance_s


def handle_final_content(third_resp: Response) -> str:
    """
    处理最终结果内容
    :param third_resp:
    :return:
    """

    third_resp.raise_for_status()

    selector = Selector(third_resp.text)

    final_content = selector.xpath('//div[@class="_j_content"]').get()

    final_content = (
            final_content or selector.xpath('//div[@class="_j_content_box"]').get()
    )
    final_content = final_content or selector.xpath('//div[@class="post_info"]').get()

    if final_content is None:
        raise Exception("爬取异常,内容为空")

    return final_content



from dataharvest.proxy.base import BaseProxy, Proxy
from dataharvest.spider import AutoSpider
from dataharvest.spider.base import SpiderConfig


class MyProxy(BaseProxy):

    def __call__(self) -> Proxy:
        return Proxy(protocol="http", host="127.0.0.1", port="53380", username="username", password="password")


def test_proxy_constructor():
    proxy_gene_func = MyProxy()
    auto_spider = AutoSpider(config=SpiderConfig(proxy_gene_func=proxy_gene_func))
    url = "https://baike.baidu.com/item/%E6%98%8E%E5%94%90%E5%AF%85%E3%80%8A%E7%81%8C%E6%9C%A8%E4%B8%9B%E7%AF%A0%E5%9B%BE%E8%BD%B4%E3%80%8B?fromModule=lemma_search-box"

    doc = auto_spider.crawl(url)
    print(doc)


def test_proxy_call():
    proxy_gene_func = MyProxy()
    auto_spider = AutoSpider()
    config = SpiderConfig(proxy_gene_func=proxy_gene_func)
    url = "https://baike.baidu.com/item/%E6%98%8E%E5%94%90%E5%AF%85%E3%80%8A%E7%81%8C%E6%9C%A8%E4%B8%9B%E7%AF%A0%E5%9B%BE%E8%BD%B4%E3%80%8B?fromModule=lemma_search-box"
    doc = auto_spider.crawl(url, config=config)
    print(doc)



from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Optional


@dataclass
class Proxy:
    protocol: str
    host: str
    port: str
    username: Optional[str] = None
    password: Optional[str] = None


class BaseProxy(ABC):
    @abstractmethod
    def __call__(self) -> Proxy:
        raise NotImplementedError



from typing import Optional

from playwright.async_api import async_playwright
from playwright.sync_api import sync_playwright
from playwright_stealth import stealth_async, stealth_sync

from dataharvest.schema import Document
from dataharvest.spider.base import SpiderConfig
from dataharvest.spider.spider import BaseSpider


class CommonSpider(BaseSpider):
    index = 2 ** 16

    def __init__(self, config: Optional[SpiderConfig] = None):
        super().__init__(config)

    def match(self, url: str) -> bool:
        return True

    def crawl(self, url: str, config: Optional[SpiderConfig] = None) -> Document:
        config = self._merge_config(config)
        with sync_playwright() as playwright:
            browser = playwright.chromium.launch(
                **self.convert_2_playwright_lunch_arg(config))
            page = browser.new_page()
            if config.headers:
                page.set_extra_http_headers(config.headers)
            stealth_sync(page)
            page.goto(url)
            page.wait_for_load_state(state="load")
            html = page.content()
            document = Document(url=page.url, metadata={}, page_content=html)
            return document

    async def a_crawl(
            self, url: str, config: Optional[SpiderConfig] = None
    ) -> Document:
        config = self._merge_config(config)
        async with async_playwright() as p:
            browser = await p.chromium.launch(
                **self.convert_2_playwright_lunch_arg(config))
            page = await browser.new_page()
            if config.headers:
                await page.set_extra_http_headers(config.headers)
            await stealth_async(page)
            await page.goto(url)
            await page.wait_for_load_state("load")
            html = await page.content()
            await browser.close()
            return Document(url=url, metadata={}, page_content=html)



import html2text
from parsel import Selector

from dataharvest.purifier.purifier import BasePurifier
from dataharvest.schema import Document


class XiaoHongShuPurifier(BasePurifier):
    def __init__(self):
        self.convertor = html2text.HTML2Text()
        # self.convertor.ignore_links = True
        self.convertor.body_width = 0

    def match(self, url: str) -> bool:
        return "/www.xiaohongshu.com/explore/" in url

    def purify(self, doc: Document) -> Document:
        selector = Selector(doc.page_content)

        content = ""
        # 标题
        title_label = selector.xpath("//div[@id='detail-title']/text()").get()
        if title_label:
            content += f"# {self.convertor.handle(title_label)}"

        # 文字
        desc_label = selector.xpath("//div[@id='detail-desc']").get()
        content += self.convertor.handle(desc_label)

        # 图片
        img_label = selector.xpath("//div[@class='swiper-wrapper']")
        img_label.xpath(
            "./div[@class='swiper-slide swiper-slide-duplicate swiper-slide-prev']").drop()
        img_label.xpath(
            "./div[@class='swiper-slide swiper-slide-duplicate swiper-slide-duplicate-active']").drop()
        if img_label:
            content += self.convertor.handle(img_label.get())

        return Document(
            url=doc.url, metadata={**doc.metadata}, page_content=content
        )



import logging


def test_tavily_searcher():
    from dataharvest.searcher.tavily_searcher import TavilySearcher

    searcher = TavilySearcher()
    logging.info(searcher.search("战国水晶杯"))


def test_sky_searcher():
    from dataharvest.searcher.sky_searcher import SkySearcher
    api_key = ""
    app_secret = ""

    searcher = SkySearcher(app_key=api_key,
                           app_secret=app_secret)
    r = searcher.search("苏州博物馆里明唐寅灌木丛绦图轴的资料")
    assert r



import copy
from typing import List, Optional

from dataharvest.schema import Document
from dataharvest.spider.base import BaseSpider, SpiderConfig


class AutoSpider:
    _spiders: List[BaseSpider] = []

    def __init__(self, config: SpiderConfig = SpiderConfig()):
        AutoSpider._spiders = [
            cls(copy.deepcopy(config)) for cls in BaseSpider.__subclasses__()
        ]
        AutoSpider._spiders.sort(key=lambda spider: spider.index)

    @classmethod
    def register(cls, spider: BaseSpider):
        AutoSpider._spiders.append(spider)
        AutoSpider._spiders.sort(key=lambda s: s.index)

    def _route(self, url: str) -> BaseSpider:
        for spider in self._spiders:
            if spider.match(url=url):
                return spider
        raise Exception(f"Cannot find spider for url: {url}")

    def crawl(self, url: str, config: Optional[SpiderConfig] = None) -> Document:
        spider = self._route(url)
        return spider.crawl(url, config)

    async def a_crawl(self, url: str,
                      config: Optional[SpiderConfig] = None) -> Document:
        spider = self._route(url)
        return await spider.a_crawl(url, config)
